#!/bin/bash

# This script is in Beta

ACTIVE_PROFILE=
LIBS_FOLDER=./libs
CONFIG_FOLDER=./
JAR_FILE=pubsubplus-connector-salesforce-1.0.0.jar
JVM_OPTS=
RUN_STRING="java"
BACKGROUND=false

CMD=()
ERRS=()

function message_pref {
  echo 'pubsubplus-connector-salesforce'
}

function usage {
  message_pref
  echo "Usage: start.sh [-l FOLDER] [-p PROFILE] [-c FOLDER] [-j FILE] [-o OPTIONS] [-b]"
  echo
  echo "To start connector use following parameters:"
  echo ""
  echo "-l | --libs        Directory containing required and optional"
  echo "                   dependency jars. Such as Micrometer metrics"
  echo "                   export dependencies (if configured)."
  echo "                   If not specified, it will use the current"
  echo "                   './libs/' folder"
  echo ""
  echo "-p | --profile     Profile to be used with connector's configuration."
  echo "                   The configuration file named "
  echo "                   'application-<profile>.yml' will be used"
  echo "                   Default value is empty, no profile is used."
  echo ""
  echo "-c | --config      Path to the folder containing the"
  echo "                   configuration files to be applied during"
  echo "                   connector startup for chosen profile."
  echo "                   By default it is set to the current folder."
  echo ""
  echo "-j | --jar         Path to specified JAR file to start connector."
  echo "                   if not specified, the default jar file is used,"
  echo "                   from the current folder"
  echo ""
  echo "-o | --options     Specifies JVM options used on Connector start."
  echo "                   example: '-Xms64M -Xmx1G'"
  echo ""
  echo "-b | --background  Runs the Connector in the background. No logs will be "
  echo "                   displayed and Connector continues running in detached"
  echo "                   mode."
  echo ""
  echo "-h | --help        Print this help page and exit"
  echo ""
}

function print_errors {
  message_pref
  echo "Connector startup failed:"
  echo
  for ERR in "${ERRS[@]}"; do
    echo "$ERR"
  done
  echo
  exit 1
}

function check_file {
  if [ ! -f "$1" ]; then
    ERRS+=("The following file doesn't exist on your filesystem:       '$1'")
  fi
}

function check_folder {
  if [ ! -d "$1" ]; then
    ERRS+=("The following folder doesn't exist on your filesystem:     '$1'")
  fi
}

function set_lib_folder {
  LIBS_FOLDER="$1"
  check_folder "$LIBS_FOLDER"
}

function set_active_profile {
  ACTIVE_PROFILE="$1"
}

function set_config_folder {
  CONFIG_FOLDER="$1"

  #adding trailing slash
  case "$1" in
  *)
    CONFIG_FOLDER="$CONFIG_FOLDER/"
    ;;
  esac

  check_folder "$CONFIG_FOLDER"
}

function set_jar_file {
  JAR_FILE="$1"
  check_file "$JAR_FILE"
}

function set_jvm_opts {
  JVM_OPTS="$1"
}

function set_background_run {
  BACKGROUND=true
}

function search_for_config_file {
  if [ -z "$ACTIVE_PROFILE" ]; then
    CONFIG_FILE="application."
  else
    CONFIG_FILE="application-$ACTIVE_PROFILE."
  fi

  #current folder
  if ls $CONFIG_FOLDER$CONFIG_FILE* 1> /dev/null 2>&1; then
    CONFIG_FOLDER=$CONFIG_FOLDER
    return
  fi

  #parent folder
  if ls $CONFIG_FOLDER"../"$CONFIG_FILE* 1> /dev/null 2>&1; then
    CONFIG_FOLDER=$CONFIG_FOLDER"../"
    return
  fi

  #current folder/config
  if ls $CONFIG_FOLDER"./config/"$CONFIG_FILE* 1> /dev/null 2>&1; then
    CONFIG_FOLDER=$CONFIG_FOLDER"config/"
    return
  fi

  #parent folder/config
  if ls $CONFIG_FOLDER"../config/"$CONFIG_FILE* 1> /dev/null 2>&1; then
    CONFIG_FOLDER=$CONFIG_FOLDER"../config/"
    return
  fi

  ERRS+=("Couldn't find configuration file "$CONFIG_FILE" for profile: $ACTIVE_PROFILE in "$CONFIG_FOLDER)
}

while [ "$1" != "" ]; do
  case $1 in
  -l | --libs)
    shift
    set_lib_folder "$1"
    ;;
  -p | --profile)
    shift
    set_active_profile "$1"
    ;;
  -c | --config)
    shift
    set_config_folder "$1"
    ;;
  -j | --jar)
    shift
    set_jar_file "$1"
    ;;
  -o | --options)
    shift
    set_jvm_opts "$1"
    ;;
  -b | --background)
    shift
    set_background_run
    ;;
  -h | --help)
    usage
    exit
    ;;
  esac
  shift
done

search_for_config_file "$CONFIG_FOLDER"

if ((${#ERRS[@]} != 0)); then
  # shellcheck disable=SC2128
  print_errors "$ERRS"
fi

BASE_RUN_STRING="-Dloader.path=$LIBS_FOLDER -Dspring.profiles.active=$ACTIVE_PROFILE -jar $JAR_FILE --spring.config.additional-location="$CONFIG_FOLDER

message_pref
echo 'Starting the pubsubplus-connector-salesforce...'

if [ "$BACKGROUND" = true ]; then
  CMD=("nohup" "$RUN_STRING" "$JVM_OPTS" "$BASE_RUN_STRING" ">/dev/null 2>&1 &")
  eval "${CMD[@]}"
  echo "The pubsubplus-connector-salesforce started with PID: "$!
  exit
else
  CMD=("$RUN_STRING" "$JVM_OPTS" "$BASE_RUN_STRING")
  eval "${CMD[@]}"
fi
